#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
Splits a VCALENDAR file (a collection of VEVENT and VTODO objects) 
in its parts, i.e., individual files containing a single VEVENT/VTODO.

The files are named based on the UID of the VEVENT/VTODO object and each 
VEVENT/VTODO is wrapped in the same VCALENDAR meta data as the original file.

Code comes from: http://cysec.org/blog/2011/06/20/splitting-a-vcalendar-into-single-vevents/
and was extended by Eric Scheibler: http://eric-scheibler.de
"""

import os, sys, argparse

def extractUID(vitem):
    """
    Extracts the UID attribute fron VEVENT/VTODO
    """
    for x in vitem:
        if x[0:3] == "UID":
            return x.split(":")[1]


def writeToFile(entry, pre, post):
    uid = extractUID(entry)
    # File name is UID of event. strip the two newline characters
    output = open(os.path.join(args.output_dir, uid.replace('/', '')[:-2] + '.ics'), 'w')
    output.write(''.join(pre))
    output.write(''.join(entry))
    output.write(''.join(post))
    output.close()

def main():
    # open VCALENDAR file 
    vcal = open(args.calendar_file, 'r').readlines()

    # Preamble
    pre = []
    # Postamble
    post = []
    # current entry
    entry = []
    # list of all VEVENTS/VTODOs
    vitems = []

    for line in vcal:
        # skip the "method" line, otherwise the cal entry is not valid
        if line.startswith("METHOD:PUBLISH"):
            continue

        # Relevant blocks start with "BEGIN:VEVENT" OR "BEGIN:VTODO"
        if line.startswith("BEGIN:VEVENT") or \
           line.startswith("BEGIN:VTODO"):

            # everything before the first VEVENT/VTODO was preamble
            if not pre:
                pre = entry
                entry = []

        entry.append(line)

        # When block is complete, store to list of vitems and start new entry
        if line.startswith("END:VEVENT") or  \
           line.startswith("END:VTODO"):

            # Add item to list of vitems
            vitems.append(entry)
            entry = []

    # everything after the last entry is postamble
    post = entry

    # output all events and todos
    for x in vitems:
        writeToFile(x, pre, post)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="split_vcalendar")
    parser.add_argument("calendar_file", nargs="?", default="", help="Calendar file to split")
    parser.add_argument("output_dir", nargs="?", default="", help="Output folder for calendar events")
    args = parser.parse_args()
    if args.calendar_file == "":
        print("Missing calendar input file")
        sys.exit(1)
    if os.path.exists(args.calendar_file) == False:
        print("Calendar file does not exist")
        sys.exit(1)
    if args.output_dir == "":
        print("Missing output folder for events")
        sys.exit(1)
    if os.path.exists(args.output_dir) == False:
        os.mkdir(args.output_dir)
    main()

